<?php
namespace Ig\IgGridgallery\Controller;

use TYPO3\CMS\Core\Page\PageRenderer;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class GridgalleryController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

    /**
     * fileRepository.
     *
     * @var \TYPO3\CMS\Core\Resource\FileRepository
     * @TYPO3\CMS\Extbase\Annotation\Inject
     */
    public $fileRepository = null;

    /**
     * metaDataRepository .
     *
     * @var \TYPO3\CMS\Core\Resource\Index\MetaDataRepository
     * @TYPO3\CMS\Extbase\Annotation\Inject
     */
    public $metaDataRepository = null;

    /**
     * The image ratio for the slider.
     *
     * @var float
     */
    private $sliderRatio = 1.0;

    public function initializeAction()
    {
        $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
        $pageRenderer->addCssFile('EXT:ig_gridgallery/Resources/Public/Css/ig_gridgallery.scss');
    }

    /**
     * Displays a gallery.
     *
     * @return void
     */
    public function showAction()
    {
        $flexForm = $this->settings['flexform'];
        $imagesUidArray = array_filter(explode(',', $flexForm['images']));
        $layer2Uids = array_filter(explode(',', $flexForm['layer2']));
        $layer3Uids = array_filter(explode(',', $flexForm['layer3']));
        $layer4Uids = array_filter(explode(',', $flexForm['layer4']));

        $multiLayer = 0;
        if(!empty($layer2Uids) || !empty($layer3Uids) || !empty($layer4Uids)){
            $multiLayer = 1;
        }

        // Get the image ratio as a float number
        if ($flexForm['aspectRatio'] != 'aspect-spezial') {
            $ratio = explode('-', str_replace('aspect', '', $flexForm['aspectRatio']));
            if ($ratio[0] != 0) {
                $this->sliderRatio = $ratio[1] / $ratio[0];
            } else {
                $ratio = 0;
            }
        }

        //$pageRenderer=GeneralUtility::makeInstance(\TYPO3\CMS\Core\Page\PageRenderer::class);
        //$pageRenderer->addJsFooterLibrary( 'slickThumbs', 'EXT:ig_render/Resources/Public/slick/thumbs.js');
        $pageRenderer = GeneralUtility::makeInstance(\Internetgalerie\IgRender\Utility\PageRendererUtility::class);
        $pageRenderer->reset();
        /**
         * We check the 'orientation' of the images. This is needed so we can later
         * define which images should have a width, or a height of 100% so they can
         * fill the container.
         */
        $orientation = [];
        $images = $this->getImages($imagesUidArray);
        if(!empty($layer2Uids)){
            $layer2 = $this->getImages($layer2Uids);
        }
        if(!empty($layer3Uids)){
            $layer3 = $this->getImages($layer3Uids);
        }

        if ($layer3) {
            $orientationLayer3 = $this->getImageOrientation($layer3Uids);
        }

        $galleryClasses = ['ig-gridgallery'];

        $animationClass = $flexForm['useHover'] ? 'item-animation' : 'item-noanimation';
        $borderClass = $flexForm['useBorder'] ? 'hasBorder' : '';

        if ($flexForm['usePreloader']) {
            array_push($galleryClasses, 'has-preloader');
        }

        if (!$flexForm['hideContent']) {
            array_push($galleryClasses, 'has-thumbs');
        }

        $this->view->assignMultiple([
            'galleryId' => 'gallery-' . uniqid(),
            'galleryClasses' => implode(' ', $galleryClasses),

            // General
            'cols' => $flexForm['cols'],
            'displayStyle' => $flexForm['displayStyle'],
            'thumbRatio' => $flexForm['thumbRatio'],
            'thumbDesign' => $flexForm['thumbDesign'],
            'images' => $images,
            'orientation' => $orientation,
            'orientationLayer2' => $orientationLayer2,
            'orientationLayer3' => $orientationLayer3,

            // Lightbox
            'useLightbox' => intval($flexForm['useLightbox']),
            'useSliderLightbox' => intval($flexForm['useSliderLightbox']),

            // Slider
            'useSlider' => intval($flexForm['slider']),
            'thumbnailMoveSlider' => intval($flexForm['thumbnailMoveSlider']),
            'transitionStyle' => $flexForm['transitionStyle'],
            'aspectRatio' => $flexForm['aspectRatio'],
            'renderMode' => $flexForm['renderMode'],
            'slideshowtime' => intval($flexForm['slideshowtime']),
            'animationtime' => intval($flexForm['animationtime']),
            'useSlideshow' => intval($flexForm['useSlideshow']),

            // Multilayer Slider
            'multiLayer' => $multiLayer,
            'layer2' => $layer2,
            'layer2Animationtime' => $flexForm['layer2Animationtime'],
            'layer3' => $layer3,
            'layer3Animationtime' => $flexForm['layer3Animationtime'],
            'layer4' => $layer4Uids,
            'layer4Animationtime' => $flexForm['layer4Animationtime'],

            // Extra
            'hideContent' => intval($flexForm['hideContent']),
            'showNav' => intval($flexForm['showNav']),
            'usePreloader' => intval($flexForm['usePreloader']),
            'animationClass' => $animationClass,
            'hasBorder' => $borderClass,
            'useDarkIcons' => intval($flexForm['useDarkIcons']),
        ]);

        // Use the entire flexform as cache identifier so configuration changes update the cache.
        $cacheIdentifier = sha1(json_encode($flexForm));

        $cacheManager = $this->objectManager->get(\TYPO3\CMS\Core\Cache\CacheManager::class);
        $cache = $cacheManager->getCache('ig_gridgallery');

        // try to find the cached content
        if (($contentAssets = $cache->get($cacheIdentifier)) === false) { // no cache content
            if (\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('ig_responsive_images')) {
                $responsiveImageUtility = GeneralUtility::makeInstance(\Ig\IgResponsiveImages\Utility\ResponsiveImagesUtility::class);
                //echo('width='.$row['width'] .','.$igFlow->getLineWidth() .'='.round($row['width']/$igFlow->getLineWidth()*100,1) );
                //$colWidth=intval(substr($flexForm['cols'],4));
                //$responsiveImageUtility->pushClass( 'igGridgallery-'. round(100/$colWidth) ); // ym-gXXX
                $responsiveImageUtility->pushClass('igGridgallery-' . $flexForm['displayStyle'] . '-' . $flexForm['cols']); // ym-gXXX
                $content = $this->view->render();
                $responsiveImageUtility->popClass();
            } else {
                $content = $this->view->render();
            }
            // sind tags uebhaupt sinnvoll?
            $this->contentObj = $this->configurationManager->getContentObject();
            $assets = $pageRenderer->getAssets();
            $contentAssets = ['content' => $content, 'assets' => $assets];
            $cache->set($cacheIdentifier, $contentAssets, ['page_' . $GLOBALS['TSFE']->id, 'element_' . $this->contentObj->data['uid']]);
        } else {
            $pageRenderer->addAssets($contentAssets['assets']);
        }

        return $contentAssets['content'];
    }

    /**
     * Displays a gallery.
     *
     * @return void
     */
    public function gridAction()
    {
        $flexForm = $this->settings['flexform'];
        $imagesUidArray = array_filter(explode(',', $flexForm['images']));
        $layer2 = array_filter(explode(',', $flexForm['layer2']));
        $layer3 = array_filter(explode(',', $flexForm['layer3']));
        $layer4 = array_filter(explode(',', $flexForm['layer4']));

        $multiLayer = 0;
        if(!empty($layer2) || !empty($layer3) || !empty($layer4)){
            $multiLayer = 1;
        }

        // Get the image ratio as a float number
        if ($flexForm['aspectRatio'] != 'aspect-spezial') {
            $ratio = explode('-', str_replace('aspect', '', $flexForm['aspectRatio']));
            if ($ratio[0] != 0) {
                $this->sliderRatio = $ratio[1] / $ratio[0];
            } else {
                $ratio = 0;
            }
        }

        //$pageRenderer=GeneralUtility::makeInstance(\TYPO3\CMS\Core\Page\PageRenderer::class);
        //$pageRenderer->addJsFooterLibrary( 'slickThumbs', 'EXT:ig_render/Resources/Public/slick/thumbs.js');
        $pageRenderer = GeneralUtility::makeInstance(\Internetgalerie\IgRender\Utility\PageRendererUtility::class);
        $pageRenderer->reset();
        /**
         * We check the 'orientation' of the images. This is needed so we can later
         * define which images should have a width, or a height of 100% so they can
         * fill the container.
         */
        $orientation = [];
        $images = $this->getImages($imagesUidArray);
        if(!empty($layer2Uids)){
            $layer2 = $this->getImages($layer2Uids);
        }
        if(!empty($layer3Uids)){
            $layer3 = $this->getImages($layer3Uids);
        }

        if ($layer3) {
            $orientationLayer3 = $this->getImageOrientation($layer3Uids);
        }

        $galleryClasses = ['ig-gridgallery'];

        // ugly fix because flexform is mostly deleted
        $animationClass = 'item-animation';

        if($flexForm['useSpacing']){
            $useSpacing = 'useSpacing';
            if($flexForm['customSpacing']){
                $customSpacing = $flexForm['customSpacing'];
                $negativeSpacing = "-".implode(' -',explode(' ',$flexForm['customSpacing']));
            }
        }

        if ($flexForm['usePreloader']) {
            array_push($galleryClasses, 'has-preloader');
        }

        if (!$flexForm['hideContent']) {
            array_push($galleryClasses, 'has-thumbs');
        }
        $this->view->assignMultiple([
            'galleryId' => 'gallery-' . uniqid(),
            'galleryClasses' => implode(' ', $galleryClasses),

            // General
            'cols' => $flexForm['cols'],
            'displayStyle' => $flexForm['displayStyle'],
            'thumbRatio' => $flexForm['thumbRatio'],
            'images' => $images,
            'orientation' => $orientation,
            'orientationLayer2' => $orientationLayer2,
            'orientationLayer3' => $orientationLayer3,

            // Lightbox
            'useLightbox' => intval($flexForm['useLightbox']),
            'useSliderLightbox' => intval($flexForm['useSliderLightbox']),

            // Slider
            'useSlider' => intval($flexForm['slider']),
            'thumbnailMoveSlider' => intval($flexForm['thumbnailMoveSlider']),
            'transitionStyle' => $flexForm['transitionStyle'],
            'aspectRatio' => $flexForm['aspectRatio'],
            'renderMode' => $flexForm['renderMode'],
            'slideshowtime' => intval($flexForm['slideshowtime']),
            'animationtime' => intval($flexForm['animationtime']),
            'useSlideshow' => intval($flexForm['useSlideshow']),

            // Multilayer Slider
            'multiLayer' => $multiLayer,
            'layer2' => $layer2,
            'layer2Animationtime' => $flexForm['layer2Animationtime'],
            'layer3' => $layer3,
            'layer3Animationtime' => $flexForm['layer3Animationtime'],
            'layer4' => $layer4Uids,
            'layer4Animationtime' => $flexForm['layer4Animationtime'],

            // Extra
            'hideContent' => intval($flexForm['hideContent']),
            'showNav' => intval($flexForm['showNav']),
            'usePreloader' => intval($flexForm['usePreloader']),
            'animationClass' => $animationClass,
            'useSpacing' => $useSpacing,
            'customSpacing' => $customSpacing,
            'negativeSpacing' => $negativeSpacing,
            'useDarkIcons' => intval($flexForm['useDarkIcons']),
        ]);

        // Use the entire flexform as cache identifier so configuration changes update the cache.
        $cacheIdentifier = sha1(json_encode($flexForm));

        $cacheManager = $this->objectManager->get(\TYPO3\CMS\Core\Cache\CacheManager::class);
        $cache = $cacheManager->getCache('ig_gridgallery');

        // try to find the cached content
        if (($contentAssets = $cache->get($cacheIdentifier)) === false) { // no cache content
            if (\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('ig_responsive_images')) {
                $responsiveImageUtility = GeneralUtility::makeInstance(\Ig\IgResponsiveImages\Utility\ResponsiveImagesUtility::class);
                //echo('width='.$row['width'] .','.$igFlow->getLineWidth() .'='.round($row['width']/$igFlow->getLineWidth()*100,1) );
                //$colWidth=intval(substr($flexForm['cols'],4));
                //$responsiveImageUtility->pushClass( 'igGridgallery-'. round(100/$colWidth) ); // ym-gXXX
                $responsiveImageUtility->pushClass('igGridgallery-' . $flexForm['displayStyle'] . '-' . $flexForm['cols']); // ym-gXXX
                $content = $this->view->render();
                $responsiveImageUtility->popClass();
            } else {
                $content = $this->view->render();
            }
            // sind tags uebhaupt sinnvoll?
            $this->contentObj = $this->configurationManager->getContentObject();
            $assets = $pageRenderer->getAssets();
            $contentAssets = ['content' => $content, 'assets' => $assets];
            $cache->set($cacheIdentifier, $contentAssets, ['page_' . $GLOBALS['TSFE']->id, 'element_' . $this->contentObj->data['uid']]);
        } else {
            $pageRenderer->addAssets($contentAssets['assets']);
        }

        return $contentAssets['content'];
    }

    /**
     * Returns the orientation classes (fill-width or fill-height) of the given
     * images.
     *
     * @param array $images
     * @return array
     */
    public function getImageOrientation(array $images)
    {
        $orientations = [];

        foreach ($images as $image) {
            $imageFile = $this->fileRepository->findFileReferenceByUid(intval($image));
            $metaData = $this->metaDataRepository->findByFile($imageFile->getOriginalFile());
            if ($flexForm['aspectRatio'] == 'aspect-spezial') {
                array_push($orientations, '');
            } else {
                if ($metaData['height'] > $metaData['width'] * $this->sliderRatio) {
                    array_push($orientations, 'fill-height');
                } else {
                    array_push($orientations, 'fill-width');
                }
            }
        }

        return $orientations;
    }


    /**
     * @todo merge with funciton getImageOrientation above
     * Returns images
     *
     * @param array $imagesUidArray
     * @return array
     */
    public function getImages(array $imagesUidArray)
    {
        $images = [];
        $orientations = [];
        foreach ($imagesUidArray as $image) {
            try {
                $file = $this->fileRepository->findFileReferenceByUid(intval($image));
                $images[] = $file;
                $meta = $this->metaDataRepository->findByFile($file->getOriginalFile());
                if ($flexForm['aspectRatio'] == 'aspect-spezial' || $this->sliderRatio == 0) {
                    array_push($orientations, '');
                } else {
                    if ($meta['height'] > $meta['width'] * $this->sliderRatio) {
                        array_push($orientations, 'fill-height');
                    } else {
                        array_push($orientations, 'fill-width');
                    }
                }
            } catch (\Exception $e) {
                // Do nothing
            }
        }

        return $images;
    }
}
